<?php
/**
 * Created by PhpStorm.
 * User: Alexander
 * Date: 20.10.2017
 * Time: 14:56
 */

namespace UBRIR;

final class Core extends CoreSmartLogger
{
    private static $instance;

    /**
     * @var InterfaceConnectionConfig
     */
    protected $ubrrCoreDir;

    private $_connector;

    //ServerConnector config
    private $serverConfigured = false;

    protected $approveUrl;
    protected $cancelUrl;
    protected $declineUrl;

    protected $shopId;
    protected $privateCertificatePass;

    protected $CACertificateFile;
    protected $publicCertificateFile;
    protected $privateCertificateFile;

    public function __construct(InterfaceConnectionConfig $connector = null)
    {
        if ($connector !== null)
            $this->_connector = $connector;

        $classesDir = realpath(dirname(__FILE__)) . '/';
        $this->ubrrCoreDir = realpath($classesDir . '/../') . '/';

        parent::__construct($this->ubrrCoreDir);
    }

    public function __destruct()
    {
        parent::__destruct();
    }

    /**
     * @return Core
     */
    public static function GetCore()
    {
        if (is_null(self::$instance)) {
            self::$instance = new Core();
        }

        return self::$instance;
    }

    public function PlugInCMSConnector(InterfaceConnectionConfig $connector)
    {
        $this->_connector = $connector;
        return $this;
    }

    /**
     * @return InterfaceConnectionConfig
     * @throws \Exception
     */
    public function GetCMSConnector()
    {
        if ($this->_connector == null)
            throw new \Exception('Модуль не подключен к CMS');

        return $this->_connector;
    }

    public function GetViewsDir()
    {
        return $this->ubrrCoreDir . 'views/';
    }

    public function IncludeTemplate($templateName, $view_data = Array())
    {
        //Выковыриваем все из массива $view_data "наружу" :)
        foreach ($view_data as $code => $value) {
            ${$code} = $value;
        }

        $viewDir = $this->GetViewsDir();
        $templateFile = $viewDir . 'ubrr.' . $templateName . '.php';
        if (!file_exists($templateFile)) {
            $this->Log('Попытка подключить несуществующий шаблон: ' . $templateName);
            return false;
        }

        ob_start();
        include_once($templateFile);
        $result = ob_get_clean();

        return $result;
    }

    public function GetShopId()
    {
        if ($this->shopId === null)
            $this->shopId = $this->_connector->GetShopId();

        return $this->shopId;
    }

    public function getPrivateCertificatePass()
    {
        if ($this->privateCertificatePass === null)
            $this->privateCertificatePass = $this->_connector->GetPrivateCertificatePassword();

        return $this->privateCertificatePass;
    }


    public function GetOrderDataByUBRIROrderID($TWPG_OrderId)
    {
        $connector = $this->GetCMSConnector();

        return $connector->GetOrderDataByUBRIROrderID($TWPG_OrderId);
    }

    /**
     * Заполняем проверяем настройки сертификатов для общения с сервером
     */
    public function ServerConnectionConfig($force = false, $publicCertificate = '', $privateCertificate = '', $privatePass = '')
    {
        if (!$this->serverConfigured || ($this->serverConfigured && $force)) {
            $connector = $this->GetCMSConnector();

            $this->shopId = $this->GetShopId();

            if (!($this->shopId > 0)) {
                $this->LogError('Не задан Shop ID');
            }

            $CACertificate = '';

            //$certFile = $this->ConvertCertificateToPEM($certFile);
            //$certFile = $UBRR_ROOT . "certs/cert.pem";

            //Сертификат банка
            if (!empty($CACertificate)) {
                //Проверка существует ли переданных сертификат
                if (!file_exists($CACertificate)) {
                    $CACertificate = false;
                }
            } else {
                //Поиск стандартного банковского сертификата
                $bankCAFile = 'bank';
                if ($connector->GetWorkMode()->IsTest()) {
                    $bankCAFile = 'bank_test';
                }

                $CACertificate = $this->ubrrCoreDir . 'certs/' . $bankCAFile . '.pem';
                if (!file_exists($CACertificate)) {
                    //Пробуем найти crt
                    $CACertificate = $this->ubrrCoreDir . 'certs/' . $bankCAFile . '.crt';
                    if (!file_exists($CACertificate)) {
                        $CACertificate = false;
                    }
                }
            }

            //Публичный сертификат
            if ($force && !empty($publicCertificate)) {
                if (!file_exists($publicCertificate)) {
                    $publicCertificate = false;
                }
            } else {
                //Поиск публичного сертификата в соответсвии с изначальными настройками
                $validPublicFiles = $this->Certificates_DefaultPublicFiles();

                $publicCertificate = false;
                foreach ($validPublicFiles as $publicCertFile)
                    if (file_exists($publicCertFile)) {
                        $publicCertificate = $publicCertFile;
                        break;
                    }
            }

            //Приватный сертификат
            if ($force && !empty($privateCertificate)) {
                if (!file_exists($privateCertificate)) {
                    $privateCertificate = false;
                }
            } else {
                //Поиск публичного сертификата в соответсвии с изначальными настройками

                $privateCertificate = false;
                $validPrivateFiles = $this->Certificates_DefaultPrivateFiles();
                foreach ($validPrivateFiles as $privateCertFile)
                    if (file_exists($privateCertFile)) {
                        $privateCertificate = $privateCertFile;

                        break;
                    }
            }

            if ($force && !empty($privatePass)) {
                $this->privateCertificatePass = $privatePass;
            } else {
                $this->privateCertificatePass = $connector->GetPrivateCertificatePassword();
            }

            $this->publicCertificateFile = $publicCertificate;
            $this->privateCertificateFile = $privateCertificate;
            $this->CACertificateFile = $CACertificate;

            $this->serverConfigured = true;
        }
    }


    public function Certificates_DefaultPrivateFiles()
    {
        $shopId = $this->GetShopId();
        if (empty($shopId)) {
            $shopId = '########';
        }

        $validPrivateFiles = Array();
        $validPrivateFiles[] = $this->ubrrCoreDir . 'certs/' . $shopId . '_private.key';
        $validPrivateFiles[] = $this->ubrrCoreDir . 'certs/' . $shopId . '_private.pem';
        $validPrivateFiles[] = $this->ubrrCoreDir . 'certs/user.key';

        return $validPrivateFiles;
    }

    public function Certificates_DefaultPublicFiles()
    {
        $shopId = $this->GetShopId();
        if (empty($shopId)) {
            $shopId = '########';
        }

        $validPublicFiles = Array();
        $validPublicFiles[] = $this->ubrrCoreDir . 'certs/' . $shopId . '.pem';
        $validPublicFiles[] = $this->ubrrCoreDir . 'certs/user.pem';

        return $validPublicFiles;
    }


    public function getApproveURL()
    {
        return $this->approveUrl;
    }

    public function getCancelURL()
    {
        return $this->cancelUrl;
    }

    public function getDeclineURL()
    {
        return $this->declineUrl;
    }

    /**
     * @return DataContainer
     */
    public function CheckSecurityConfiguration()
    {
        if (empty($this->shopId) || empty($this->publicCertificateFile) || empty($this->privateCertificateFile) || empty($this->CACertificateFile)) {
            $response = new DataContainer(false);

            if (empty($this->shopId))
                $response->AddError('Оплата не настроена. Обратитесь к администратору магазина');

            if (empty($this->publicCertificateFile)) {
                $validPublicFiles = $this->Certificates_DefaultPublicFiles();
                foreach ($validPublicFiles as $publicCertFile)
                    $response->AddError('Не найден файл публичного ключа [' . $publicCertFile . ']');
            }

            if (empty($this->privateCertificateFile)) {
                $validPrivateFiles = $this->Certificates_DefaultPrivateFiles();
                foreach ($validPrivateFiles as $privateCertFile)
                    $response->AddError('Не найден файл приватного ключа [' . $privateCertFile . ']');
            }
            if (empty($this->CACertificateFile))
                $response->AddError('Не найден файл банковского сертификата для связи с платежным шлюзом');

        } else {
            //Можно тут так же проверять на существования файлов - пока специально не делаем проверяя более глубокие ошибки CURL

            $response = new DataContainer(true);
        }

        return $response;
    }

    /**
     * @return WorkMode
     */
    public function getWorkMode()
    {
        $connector = $this->GetCMSConnector();

        return $connector->GetWorkMode();
    }

    public function getCACertificateFile()
    {
        if ($this->CACertificateFile === null)
            $this->ServerConnectionConfig();

        return $this->CACertificateFile;
    }

    public function getPublicCertificateFile()
    {
        if (!$this->serverConfigured)
            $this->ServerConnectionConfig();

        return $this->publicCertificateFile;
    }

    public function getPrivateCertificateFile()
    {
        if (!$this->serverConfigured)
            $this->ServerConnectionConfig();

        return $this->privateCertificateFile;
    }

    public function GetCertificateGenerator()
    {
        return new NewCertificates($this);
    }
}