<?php

namespace UBRIR;

class GatewayInteractionPanel
{
    public function __construct(InterfaceConnectionConfig $connector = null, $useLogs = true)
    {
        parent::__construct($connector);
    }

    public function ConnectToCMS(InterfaceConnectionConfig $connector)
    {
        $this->PlugInCMSConnector($connector);
        return $this;
    }

    //Заполнение необходимых параметров и затем попытка передачи
    public function SetupAndPrepareToPay($orderId, $orderAmount, $approveCallbackURL, $cancelCallbackURL = '', $declineCallbackURL = '')
    {
        if (empty($cancelCallbackURL))
            $cancelCallbackURL = $approveCallbackURL;

        if (empty($declineCallbackURL))
            $declineCallbackURL = $approveCallbackURL;

        $this->approveUrl = $approveCallbackURL;
        $this->cancelUrl = $cancelCallbackURL;
        $this->declineUrl = $declineCallbackURL;

        //умножаем на 100, потому что передача в платежный шлюз в копейках
        $orderAmount = $orderAmount * 100;

        $serverResponse = ServerConnector::PrepareToPay($this, $orderId, $orderAmount);

        return $serverResponse;
    }

    //region Работа с сертификатами
    protected function ConvertCertificateToPEM($cerFile)
    {
        $certificateFilePath = substr($cerFile, 0, strrpos($cerFile, '.'));
        $certificatePEMFile = $certificateFilePath . '.pem';

        if (!file_exists($certificatePEMFile)) {
            if (file_exists($cerFile)) {
                $certificateCAcerContent = file_get_contents($cerFile);

                /* Convert .cer to .pem, cURL uses .pem */
                $certificateCApemContent = '-----BEGIN CERTIFICATE-----' . PHP_EOL
                    . chunk_split(base64_encode($certificateCAcerContent), 64, PHP_EOL)
                    . '-----END CERTIFICATE-----' . PHP_EOL;

                file_put_contents($certificatePEMFile, $certificateCApemContent);
            }
        }

        if (file_exists($certificatePEMFile)) {
            return $certificatePEMFile;
        }
        return false;
    }

    //endregion

    public function ShowPaymentError($continueLink, $errorMessage = '', PaymentResultViewer $resultViewer = null)
    {
        $connector = $this->GetCMSConnector();

        if (empty($continueLink))
            $continueLink = $connector->GetDefaultContinueLink();

        if ($resultViewer == null)
            $resultViewer = new PaymentResultViewer();

        if (!empty($errorMessage))
            $resultViewer->SetFailureMessage($errorMessage);

        global $view_data;
        $view_data = $resultViewer
            ->SetContinueLink($continueLink)
            ->ViewData();

        $viewDir = $this->GetViewsDir();
        $templateFile = $viewDir . '/ubrr.view.paymentstatus.php';
        include_once($templateFile);
    }

    /**
     * Сохраняем данные о заказе и показываем страницу статуса оплаты
     * @param $continueLink
     * @param string $message
     * @param PaymentResultViewer|null $resultData
     */
    public function ShowPaymentSuccess($continueLink, $message = '', PaymentResultViewer $resultData = null)
    {
        $connector = $this->GetCMSConnector();

        if (empty($continueLink))
            $continueLink = $connector->GetDefaultContinueLink();

        if ($resultData == null)
            $resultData = new PaymentResultViewer();

        global $view_data;
        $view_data = $resultData
            ->SetSuccessMessage($message)
            ->SetContinueLink($continueLink)
            ->ViewData();

        $viewDir = $this->GetViewsDir();
        $templateFile = $viewDir . '/ubrr.view.paymentstatus.php';
        include_once($templateFile);
    }
}