<?php

use ISYS\UBRIR\OpencartAdmin;
use ISYS\UBRIR\OpencartPayment;
use ISYS\UBRIR\OpencartUBRiRConnector;
use UBRIR\Core;
use UBRIR\NewCertificates;

$currentDir = dirname(__FILE__) . '/';
include_once($currentDir . 'classes/_autoload.php');
include_once($currentDir . 'ubrr.core/_autoload.php');

class ControllerExtensionPaymentUbrr extends Controller
{
    private $connector;
    private $error = array();
    private $version = '2.0';

    public function index()
    {
        $this->connector = new OpencartUBRiRConnector($this);

        $this->CheckDBStructure();
        $data = $this->LoadLocalizationData();
        $this->document->setTitle($this->language->get('heading_title'));

        $data['action'] = $this->makeUrl('extension/payment/ubrr');
        $data['cancel'] = $this->makeUrl('extension/extension', 'type=payment');
        $data['terms'] = '';
        $data['diagnosticReport'] = '';
        $data['ubrir_css_url'] = '';
        $data['module_css_url'] = '';

        $data = array_merge($data, $this->ParseFormData());
        $data['ajax_response'] = $this->CheckAjaxRequests();

        if (empty($data['ajax_response'])) { //Небольшая оптимизация, если возрвращаем ajax - то нам нет нужды перерисовывать остальные части админки
            if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
                //Не используем штатный способ сохранения параметрво потому у нас несколько форм на разных закладках + есть свои скрытые поля
                //$this->load->model('setting/setting');
                //$this->model_setting_setting->editSetting('ubrr', $this->request->post);
                $this->connector->SaveConfigFormParams($this->request->post);
                $this->session->data['success'] = sprintf($this->language->get('text_success'), $this->language->get('heading_title'));
                $this->response->redirect($this->makeUrl('extension/extension', 'type=payment'));
            }

            $data['header'] = $this->load->controller('common/header');
            $data['column_left'] = $this->load->controller('common/column_left');
            $data['footer'] = $this->load->controller('common/footer');

            $cmsModule = new OpencartAdmin($this);
            $data['cms_module'] = $cmsModule;

            //region UBRIR Check

            $core = Core::GetCore();
            $cmsNavigator = new \ISYS\UBRIR\OpencartNavigator();
            $ubrirDiagnostic = new \UBRIR\Diagnostics($this->connector);
            $ubrirDiagnostic->UseNavigator($cmsNavigator);

            $data['diagnosticReport'] = $ubrirDiagnostic->GetDiagnosticPage();
            $data['ubrir_css_url'] = substr($core->GetViewsDir() . 'css/ubrir.css', strlen($_SERVER['DOCUMENT_ROOT']));

            $stylesFile = dirname(__FILE__) . '/views/css/ubrir_opencart.css';
            $data['module_css_url'] = substr($stylesFile, strlen($_SERVER['DOCUMENT_ROOT']));

            $data['action_clean_debug_log'] = $this->makeUrl('extension/payment/ubrr/clean_debug_log');
            //      $data['cms_module'] = $cmsModule;
        } else {
            $data['header'] = '';
            $data['column_left'] = '';
            $data['footer'] = '';
        }

        $this->response->setOutput($this->load->view('extension/payment/ubrr', $data));
    }

    protected function CheckDBStructure()
    {
        //TODO: можно хранить в конфиге что структура была проверена - он все равно каждый раз собирается из БД, чтобы не делать дополнительные запросы
        $this->db->query("CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "twpg_orders` (`shoporderid` VARCHAR(11), `OrderID`
                INT(11) NOT NULL,`SessionID` VARCHAR(40) NOT NULL) ENGINE=InnoDB DEFAULT CHARSET=utf8");
    }

    protected function CheckAjaxRequests()
    {
        $ajax_response = '';

        $ajaxType = '';
        if (isset($_REQUEST['ajax_type']))
            $ajaxType = $_REQUEST['ajax_type'];

        switch ($ajaxType) {
            case 'tab_refresh': {
                global $result, $openCartController;

                $openCartController = $this;

                $result = 'ooups..';

                $ajaxDir = dirname(__FILE__) . '/ajax/';
                $files = scandir($ajaxDir);
                $tabId = $_REQUEST['tab_id'];
                $ajax_page = substr($tabId, strlen('#tab_')) . '.php';
                if (in_array($ajax_page, $files)) {
                    include_once($ajaxDir . $ajax_page);
                }

                return $result;
            }
                break;

            case 'clear_log': {
                $panelAdmin = new OpencartAdmin($this);
                $data['debug_log'] = $panelAdmin->CleanLog();
                return $panelAdmin->ReturnHTMLLog();
            }
                break;
            case 'certificate_generate': {

                //$connector->SetBankReceiversEmailList($_REQUEST['emailTo']);
                $panel = new OpencartPayment();
                $panel->ConnectToCMS($this->connector);
                $cert = new NewCertificates($panel);

                $generated = $cert->Generate($_REQUEST['merchantId'], $_REQUEST['emailFrom'], $_REQUEST['keyPass']);
                if ($generated->isSuccess()) {
                    $ajax_response = '<h2>Запрос на выпуск сертификата успешно выполнен. Ожидайте ответа из банка на почту указанную в форме генерации запроса!</h2>';
                } else {
                    $ajax_response = $generated->arMessages['ERROR'][0];
                }
            }
                break;
        }

        return $ajax_response;
    }

    protected
    function LoadLocalizationData()
    {
        $data = Array();

        $this->load->language('extension/payment/ubrr');
        $this->load->model('localisation/language');
        $this->load->model('localisation/order_status');

        $data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

        $data['lang'] = $this->language;

        //Загрузка названия для цепочки навигации
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->makeUrl('common/dashboard')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_payment'),
            'href' => $this->makeUrl('extension/extension', 'type=payment')
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->makeUrl('extension/payment/ubrr')
        );

        return $data;
    }

    protected function ParseFormData()
    {
        $arDefaultConfig = OpencartUBRiRConnector::GetDefaultConfig();

        $data = Array();
        foreach ($arDefaultConfig as $code => $defaultValue) {
            if (isset($this->request->post[$code])) {
                $data[$code] = $this->request->post[$code];
            } else {
                $data[$code] = $this->config->get($code);
            }
        }

        $data['error_ubrr_merchant_id'] = '';
        if (isset($this->request->post['ubrr_merchant_id'])) {
            $data['ubrr_merchant_id'] = $this->request->post['ubrr_merchant_id'];
        } else {
            if (empty($this->config->get('ubrr_merchant_id'))) {
                $data['error_ubrr_merchant_id'] = $this->language->get('error_ubrr_merchant_id');
            }
            $data['ubrr_certificate_email_receivers'] = $this->config->get('ubrr_certificate_email_receivers');
        }

        $data['error_ubrr_certificate_email_receivers'] = '';
        if (isset($this->request->post['ubrr_certificate_email_receivers'])) {
            $data['ubrr_certificate_email_receivers'] = $this->request->post['ubrr_certificate_email_receivers'];
        } else {
            if (empty($this->config->get('ubrr_certificate_email_receivers'))) {
                $data['error_ubrr_certificate_email_receivers'] = $this->language->get('error_ubrr_certificate_email_receivers');
            }
            $data['ubrr_certificate_email_receivers'] = $this->config->get('ubrr_certificate_email_receivers');
        }

        return $data;
    }

    protected
    function validate()
    {
        if (!$this->validatePermission()) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        return !$this->error;
    }

    protected
    function _updateData($keys, $info = array())
    {
        $data = array();
        foreach ($keys as $key) {
            if (isset($this->request->post[$key])) {
                $data[$key] = $this->request->post[$key];
            } elseif ($this->config->get($key)) {
                $data[$key] = $this->config->get($key);
            } elseif (isset($info[$key])) {
                $data[$key] = $info[$key];
            } else {
                $data[$key] = null;
            }
        }
        return $data;
    }

    protected
    function validatePermission()
    {
        return $this->user->hasPermission('modify', 'extension/payment/ubrr');
    }

    protected
    function makeUrl($route, $url = '')
    {
        return str_replace('&amp;', '&', $this->url->link($route, $url . '&token=' . $this->session->data['token'], 'SSL'));
    }
}

?>